<?php

if (!defined('WPVIVID_PLUGIN_DIR'))
{
    die;
}

define('WPVIVID_BACKUP_TYPE_DB','backup_db');
define('WPVIVID_BACKUP_TYPE_THEMES','backup_themes');
define('WPVIVID_BACKUP_TYPE_PLUGIN','backup_plugin');
define('WPVIVID_BACKUP_TYPE_UPLOADS','backup_uploads');
define('WPVIVID_BACKUP_TYPE_UPLOADS_FILES','backup_uploads_files');
define('WPVIVID_BACKUP_TYPE_UPLOADS_FILES_OTHER','backup_uploads_files_other');
define('WPVIVID_BACKUP_TYPE_CONTENT','backup_content');
define('WPVIVID_BACKUP_TYPE_CORE','backup_core');
define('WPVIVID_BACKUP_TYPE_OTHERS','backup_others');
define('WPVIVID_BACKUP_TYPE_MERGE','backup_merge');

class WPvivid_Backup_Task
{
    private $task;

    public function __construct($task_id=false,$task=false)
    {
        if($task_id!==false)
        {
            $this->task=WPvivid_taskmanager::get_task($task_id);
        }

        if($task!==false)
        {
            $this->task=$task;
        }
    }

    public function get_id()
    {
        return $this->task['id'];
    }

    public function new_backup_task($options,$type,$action='backup')
    {
        $id=uniqid('wpvivid-');
        $this->task=false;
        $this->task['id']=$id;
        $this->task['action']=$action;
        $this->task['type']=$type;

        $this->task['status']['start_time']=time();
        $this->task['status']['run_time']=time();
        $this->task['status']['timeout']=time();
        $this->task['status']['str']='ready';
        $this->task['status']['resume_count']=0;

        if(isset($options['remote']))
        {
            if($options['remote']=='1')
            {
                if(isset($options['remote_options']))
                {
                    $this->task['options']['remote_options']=$options['remote_options'];
                }
                else
                {
                    $this->task['options']['remote_options']=WPvivid_Setting::get_remote_options();
                }

            }
            else {
                $this->task['options']['remote_options']=false;
            }
        }
        else
        {
            $this->task['options']['remote_options']=false;
        }

        if(isset($options['local']))
        {
            if($options['local']=='1')
            {
                $this->task['options']['save_local']=1;
            }
            else
            {
                $this->task['options']['save_local']=0;
            }
        }
        else
        {
            $this->task['options']['save_local']=1;
        }

        if(isset($options['lock']))
        {
            $this->task['options']['lock']=$options['lock'];
        }
        else
        {
            $this->task['options']['lock']=0;
        }

        if(isset($options['ismerge']))
        {
            if($options['ismerge']=='1')
            {
                $this->task['options']['backup_options']['ismerge']=1;
            }
            else {
                $this->task['options']['backup_options']['ismerge']=0;
            }
        }
        else {
            $this->task['options']['backup_options']['ismerge']=1;
        }

        $home_url_prefix=get_home_url();
        $home_url_prefix=$this->parse_url_all($home_url_prefix);

        $general_setting=WPvivid_Setting::get_setting(true, "");
        if(!isset($general_setting['options']['wpvivid_common_setting']['domain_include'])){
            $this->task['options']['file_prefix']=$home_url_prefix.'_'.$this->task['id'].'_'.date('Y-m-d-H-i',$this->task['status']['start_time']);
        }
        else{
            if($general_setting['options']['wpvivid_common_setting']['domain_include']){
                $this->task['options']['file_prefix']=$home_url_prefix.'_'.$this->task['id'].'_'.date('Y-m-d-H-i',$this->task['status']['start_time']);
            }
            else{
                $this->task['options']['file_prefix']=$this->task['id'].'_'.date('Y-m-d-H-i',$this->task['status']['start_time']);
            }
        }
        $this->task['options']['log_file_name']=$id.'_backup';
        $log=new WPvivid_Log();
        $log->CreateLogFile($this->task['options']['log_file_name'],'no_folder','backup');
        //$log->WriteLog(get_home_path(),'test');
        $this->task['options']['backup_options']['prefix']=$this->task['options']['file_prefix'];
        $this->task['options']['backup_options']['compress']=WPvivid_Setting::get_option('wpvivid_compress_setting');
        $this->task['options']['backup_options']['dir']=WPvivid_Setting::get_backupdir();
        $this->task['options']['backup_options']['backup']=array();

        if(isset($options['backup_files']))
        {
            if($options['backup_files']=='files+db')
            {
                $this->set_backup(WPVIVID_BACKUP_TYPE_DB);
                $this->set_backup(WPVIVID_BACKUP_TYPE_THEMES);
                $this->set_backup(WPVIVID_BACKUP_TYPE_PLUGIN);
                $general_setting=WPvivid_Setting::get_setting(true, "");
                if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                    if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS_FILES);
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS_FILES_OTHER);
                    }
                    else{
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS);
                    }
                }
                else{
                    $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS);
                }
                $this->set_backup(WPVIVID_BACKUP_TYPE_CONTENT);
                $this->set_backup(WPVIVID_BACKUP_TYPE_CORE);
            }
            else if($options['backup_files']=='files')
            {
                $this->set_backup(WPVIVID_BACKUP_TYPE_THEMES);
                $this->set_backup(WPVIVID_BACKUP_TYPE_PLUGIN);
                $general_setting=WPvivid_Setting::get_setting(true, "");
                if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                    if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS_FILES);
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS_FILES_OTHER);
                    }
                    else{
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS);
                    }
                }
                else{
                    $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS);
                }
                $this->set_backup(WPVIVID_BACKUP_TYPE_CONTENT);
                $this->set_backup(WPVIVID_BACKUP_TYPE_CORE);
            }
            else if($options['backup_files']=='db')
            {
                $this->set_backup(WPVIVID_BACKUP_TYPE_DB);
            }
            else{
                $this->task['options']['backup_options']['backup'] = apply_filters('wpvivid_set_backup_type', $options, $this->task);
                if(empty($this->task['options']['backup_options']['backup'])){
                    if($options['backup_files']['backup_db'] == '1'){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_DB);
                    }
                    if($options['backup_files']['backup_themes'] == '1'){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_THEMES);
                    }
                    if($options['backup_files']['backup_plugin'] == '1'){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_PLUGIN);
                    }
                    if($options['backup_files']['backup_uploads'] == '1'){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_UPLOADS);
                    }
                    if($options['backup_files']['backup_content'] == '1'){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_CONTENT);
                    }
                    if($options['backup_files']['backup_core'] == '1'){
                        $this->set_backup(WPVIVID_BACKUP_TYPE_CORE);
                    }
                }
            }
        }
        $this->task['data']['doing']='backup';
        $this->task['data']['backup']['doing']='';
        $this->task['data']['backup']['finished']=0;
        $this->task['data']['backup']['progress']=0;
        $this->task['data']['backup']['job_data']=array();
        $this->task['data']['backup']['sub_job']=array();
        $this->task['data']['backup']['db_size']='0';
        $this->task['data']['backup']['files_size']['sum']='0';
        $this->task['data']['upload']['doing']='';
        $this->task['data']['upload']['finished']=0;
        $this->task['data']['upload']['progress']=0;
        $this->task['data']['upload']['job_data']=array();
        $this->task['data']['upload']['sub_job']=array();
        WPvivid_Setting::update_task($id,$this->task);
        $ret['result']='success';
        $ret['task_id']=$this->task['id'];
        $log->CloseFile();
        return $ret;
    }

    private function parse_url_all($url)
    {
        $parse = parse_url($url);
        $path=str_replace('/','_',$parse['path']);
        return $parse['host'].$path;
    }

    public function set_backup($backup)
    {
        if(is_string($backup))
        {
            if(!function_exists('get_home_path'))
                require_once(ABSPATH . 'wp-admin/includes/file.php');
            $backup_data['key']=$backup;
            $backup_data['result']=false;
            $backup_data['compress']=$this->task['options']['backup_options']['compress'];
            $backup_data['finished']=0;
            $backup_data['path']=WP_CONTENT_DIR.DIRECTORY_SEPARATOR. $this->task['options']['backup_options']['dir'].DIRECTORY_SEPARATOR;
            if($backup==WPVIVID_BACKUP_TYPE_DB)
            {
                $backup_data['root_path']=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->task['options']['backup_options']['dir'];
                $backup_data['dump_db']=1;
                $backup_data['sql_file_name']=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->task['options']['backup_options']['dir'].DIRECTORY_SEPARATOR.$this->get_prefix().'_backup_db.sql';
                $backup_data['json_info']['dump_db']=1;
                $backup_data['prefix']=$this->get_prefix().'_backup_db';
                $this->task['options']['backup_options']['backup'][$backup_data['key']]=$backup_data;
            }
            else if($backup==WPVIVID_BACKUP_TYPE_THEMES)
            {
                $backup_data['root_path']=WP_CONTENT_DIR;
                $backup_data['prefix']=$this->get_prefix().'_backup_themes';
                $backup_data['files_root']=$this->transfer_path(get_theme_root());
                $backup_data['exclude_regex']=array();
                $backup_data['include_regex']=array();
                $this->task['options']['backup_options']['backup'][$backup_data['key']]=$backup_data;
            }
            else if($backup==WPVIVID_BACKUP_TYPE_PLUGIN)
            {
                $backup_data['root_path']=WP_CONTENT_DIR;
                $backup_data['prefix']=$this->get_prefix().'_backup_plugin';
                $backup_data['files_root']=$this->transfer_path(WP_PLUGIN_DIR);
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_PLUGIN_DIR.DIRECTORY_SEPARATOR.'wpvivid-backuprestore'), '/').'#';
                $backup_data['exclude_regex']=$exclude_regex;
                $backup_data['include_regex']=array();
            }
            else if($backup==WPVIVID_BACKUP_TYPE_UPLOADS)
            {
                $backup_data['root_path']=WP_CONTENT_DIR;
                $backup_data['prefix']=$this->get_prefix().'_backup_uploads';
                $upload_dir = wp_upload_dir();
                $backup_data['files_root']=$this -> transfer_path($upload_dir['basedir']);
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR.'backwpup', '/').'#';  // BackWPup backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR.'backup-guard', '/').'#';  // Wordpress Backup and Migrate Plugin backup directory
                $backup_data['exclude_regex']=$exclude_regex;
                $backup_data['include_regex']=array();
            }
            else if($backup==WPVIVID_BACKUP_TYPE_UPLOADS_FILES)
            {
                $backup_data['root_path']=WP_CONTENT_DIR;
                $backup_data['prefix']=$this->get_prefix().'_backup_uploads';
                $upload_dir = wp_upload_dir();
                $backup_data['files_root']=$this -> transfer_path($upload_dir['basedir']);
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR.'backwpup', '/').'#';  // BackWPup backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR.'backup-guard', '/').'#';  // Wordpress Backup and Migrate Plugin backup directory
                //$exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR, '/').'[0-9]{4}#';
                $backup_data['exclude_regex']=$exclude_regex;
                $backup_data['include_regex'][]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR, '/').'[0-9]{4}#';
            }
            else if($backup==WPVIVID_BACKUP_TYPE_UPLOADS_FILES_OTHER)
            {
                $backup_data['root_path']=WP_CONTENT_DIR;
                $backup_data['prefix']=$this->get_prefix().'_backup_uploads_other';
                $upload_dir = wp_upload_dir();
                $backup_data['files_root']=$this -> transfer_path($upload_dir['basedir']);
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR.'backwpup', '/').'#';  // BackWPup backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR.'backup-guard', '/').'#';  // Wordpress Backup and Migrate Plugin backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']).DIRECTORY_SEPARATOR, '/').'[0-9]{4}#';
                $backup_data['exclude_regex']=$exclude_regex;
                $backup_data['include_regex']=array();
            }
            else if($backup==WPVIVID_BACKUP_TYPE_CONTENT)
            {
                $backup_data['root_path']=get_home_path();
                $backup_data['prefix']=$this->get_prefix().'_backup_content';
                $backup_data['files_root']=$this -> transfer_path(WP_CONTENT_DIR);
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'updraft', '/').'#';   // Updraft Plus backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'ai1wm-backups', '/').'#'; // All-in-one WP migration backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'backups', '/').'#'; // Xcloner backup directory
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'upgrade', '/').'#';
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'wpvivid', '/').'#';
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.WPvivid_Setting::get_backupdir(), '/').'#';
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'plugins', '/').'#';
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path(WP_CONTENT_DIR).DIRECTORY_SEPARATOR.'cache', '/').'#';
                $upload_dir = wp_upload_dir();
                $exclude_regex[]='#^'.preg_quote($this -> transfer_path($upload_dir['basedir']), '/').'#';
                $exclude_regex[]='#^'.preg_quote($this->transfer_path(get_theme_root()), '/').'#';
                $backup_data['exclude_regex']=$exclude_regex;
                $backup_data['include_regex']=array();
            }
            else if($backup==WPVIVID_BACKUP_TYPE_CORE)
            {
                $backup_data['root_path']=get_home_path();
                $backup_data['prefix']=$this->get_prefix().'_backup_core';
                $backup_data['files_root']=$this -> transfer_path(get_home_path());
                $backup_data['json_info']['include_path'][]='wp-includes';
                $backup_data['json_info']['include_path'][]='wp-admin';
                $backup_data['json_info']['wp_core']=1;
                $backup_data['json_info']['home_url']=home_url();
                $include_regex[]='#^'.preg_quote($this -> transfer_path(get_home_path().DIRECTORY_SEPARATOR.'wp-admin'), '/').'#';
                $include_regex[]='#^'.preg_quote($this->transfer_path(get_home_path().DIRECTORY_SEPARATOR.'wp-includes'), '/').'#';
                $exclude_regex[]='#^'.preg_quote($this->transfer_path(get_home_path().DIRECTORY_SEPARATOR.'wp-admin'.DIRECTORY_SEPARATOR), '/').'pclzip-.*\.tmp#';
                $exclude_regex[]='#^'.preg_quote($this->transfer_path(get_home_path().DIRECTORY_SEPARATOR.'wp-admin'.DIRECTORY_SEPARATOR), '/').'pclzip-.*\.gz#';
                $backup_data['exclude_regex']=$exclude_regex;
                $backup_data['include_regex']=$include_regex;
            }
            else if($backup==WPVIVID_BACKUP_TYPE_MERGE)
            {
                $backup_data['root_path']=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->task['options']['backup_options']['dir'];
                $backup_data['prefix']=$this->get_prefix().'_backup_all';
                $backup_data['files']=array();
                foreach ($this->task['options']['backup_options']['backup'] as $backup_finished_data)
                {
                    $backup_data['files']=array_merge($backup_data['files'],$this->get_backup_file($backup_finished_data['key']));
                }
                $backup_data['json_info']['has_child']=1;
                $backup_data['json_info']['home_url']=home_url();
            }
            else
            {
                $backup_data=false;
            }
            if($backup_data!==false)
            {
                add_filter('wpvivid_set_backup', array($this, 'wpvivid_set_backup'),10);
                $backup_data=apply_filters('wpvivid_set_backup',$backup_data);
                $this->task['options']['backup_options']['backup'][$backup_data['key']]=$backup_data;
            }
        }
    }

    public function wpvivid_set_backup($backup_data)
    {
        if($backup_data['key']==WPVIVID_BACKUP_TYPE_PLUGIN)
        {
            $general_setting=WPvivid_Setting::get_setting(true, "");
            if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                    $backup_data['resume_packages'] = 1;
                }
            }
        }
        else if($backup_data['key']==WPVIVID_BACKUP_TYPE_MERGE)
        {
            $backup_data['resume_packages'] = 1;
        }
        else if($backup_data['key']==WPVIVID_BACKUP_TYPE_UPLOADS_FILES)
        {
            $general_setting=WPvivid_Setting::get_setting(true, "");
            if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                    $backup_data['resume_packages'] = 1;
                }
            }
        }

        return $backup_data;
    }

    public function get_need_backup_files($backup_data)
    {
        if(isset($backup_data['files'])&&!empty($backup_data['files']))
        {
            return $backup_data['files'];
        }
        else
        {
            return $this->get_file_list($backup_data['files_root'],$backup_data['exclude_regex'],$backup_data['include_regex'],$this->task['options']['backup_options']['compress']['exclude_file_size']);
        }
    }

    public function get_backup_file($key)
    {
        $files=array();
        if(array_key_exists($key,$this->task['options']['backup_options']['backup']))
        {
            $backup=$this->task['options']['backup_options']['backup'][$key];
            if($backup['finished'])
            {
                if($backup['result']!=false)
                {
                    foreach ($backup['result']['files'] as $file_data)
                    {
                        $files[]=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->task['options']['backup_options']['dir'].DIRECTORY_SEPARATOR.$file_data['file_name'];
                    }
                }
            }
        }
        return $files;
    }

    public function get_need_backup()
    {
        $this->task=WPvivid_taskmanager::get_task($this->task['id']);
        $backup=false;
        $i_finished_backup_count=0;
        $i_count_of_backup=sizeof($this->task['options']['backup_options']['backup']);
        foreach ($this->task['options']['backup_options']['backup'] as $key=>$backup_data)
        {
            if($backup_data['result']!==false)
            {
                $ret=$backup_data['result'];
                if($ret['result']!==WPVIVID_SUCCESS)
                {
                    return false;
                }
            }

            if( $backup_data['finished']==0)
            {
                if($backup===false)
                {
                    add_filter('wpvivid_get_need_backup_files', array($this, 'wpvivid_get_need_backup_files'),10);
                    $backup_data=apply_filters('wpvivid_get_need_backup_files',$backup_data);

                    $backup=$backup_data;
                }
            }
            else {
                $i_finished_backup_count++;
            }
        }

        if($i_count_of_backup>0)
        {
            $i_progress=intval(1/$i_count_of_backup*100);
            WPvivid_taskmanager::update_backup_main_task_progress($this->task['id'],'backup',$i_progress*$i_finished_backup_count,0);

            if($i_finished_backup_count>=$i_count_of_backup)
            {
                if($this->task['options']['backup_options']['ismerge']==1)
                {
                    if(!array_key_exists(WPVIVID_BACKUP_TYPE_MERGE,$this->task['options']['backup_options']['backup']))
                    {
                        $this->set_backup(WPVIVID_BACKUP_TYPE_MERGE);
                        WPvivid_Setting::update_task($this->task['id'],$this->task);
                        $backup=$this->task['options']['backup_options']['backup'][WPVIVID_BACKUP_TYPE_MERGE];
                    }
                }
            }
        }
        return $backup;
    }

    public function wpvivid_get_need_backup_files($backup_data)
    {
        if(!isset($backup_data['dump_db']))
        {
            if($backup_data['key']==WPVIVID_BACKUP_TYPE_PLUGIN)
            {
                $general_setting=WPvivid_Setting::get_setting(true, "");
                if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                    if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                        $backup_data['files']=$this->get_need_backup_folder($backup_data);
                        $this->task['options']['backup_options']['backup'][$backup_data['key']]=$backup_data;
                        WPvivid_Setting::update_task($this->task['id'],$this->task);
                    }
                }
                else{
                    $backup_data['files']=$this->get_need_backup_files($backup_data);
                }
            }
            else if ($backup_data['key']==WPVIVID_BACKUP_TYPE_UPLOADS_FILES)
            {
                $general_setting=WPvivid_Setting::get_setting(true, "");
                if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                    if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                        $backup_data['files']=$this->get_need_uploads_backup_folder($backup_data);
                        $this->task['options']['backup_options']['backup'][$backup_data['key']]=$backup_data;
                        WPvivid_Setting::update_task($this->task['id'],$this->task);
                    }
                }
                else{
                    $backup_data['files']=$this->get_need_backup_files($backup_data);
                }
            }
            else {
                $backup_data['files']=$this->get_need_backup_files($backup_data);
            }
        }
        return $backup_data;
    }

    public function update_backup_result($backup_data,$result)
    {
        $this->task=WPvivid_taskmanager::get_task($this->task['id']);

        if(array_key_exists($backup_data['key'],$this->task['options']['backup_options']['backup']))
        {
            $this->task['options']['backup_options']['backup'][$backup_data['key']]['finished']=1;

            add_filter('wpvivid_backup_update_result', array($this, 'wpvivid_backup_update_result'),10,2);
            $result=apply_filters('wpvivid_backup_update_result',$result,$backup_data);
            $this->task['options']['backup_options']['backup'][$backup_data['key']]['result']=$result;

            WPvivid_taskmanager::update_task_options($this->task['id'],'backup_options', $this->task['options']['backup_options']);

            if($result['result']==WPVIVID_FAILED)
            {
                WPvivid_taskmanager::update_backup_task_status($this->task['id'],false,'error',false,false,$result['error']);
            }
        }

        $i_finished_backup_count=0;
        $i_count_of_backup=sizeof($this->task['options']['backup_options']['backup']);

        foreach ($this->task['options']['backup_options']['backup'] as $backup_data)
        {
            if( $backup_data['finished']==1)
            {
                $i_finished_backup_count++;
            }
        }

        if($i_finished_backup_count>=$i_count_of_backup)
        {
            WPvivid_taskmanager::update_backup_main_task_progress($this->task['id'],'backup',100,1);
        }
        WPvivid_Setting::update_task($this->task['id'],$this->task);
    }

    public function wpvivid_backup_update_result($result,$backup_data)
    {
        if($result['result']==WPVIVID_SUCCESS)
        {
            $exist_files_name=array();
            foreach ($result['files'] as $temp_file_data)
            {
                $exist_files_name[$temp_file_data['file_name']]=$temp_file_data['file_name'];
            }
            if(WPVIVID_BACKUP_TYPE_PLUGIN==$backup_data['key'])
            {
                $packages_files_info = $this->get_packages_files_info(WPVIVID_BACKUP_TYPE_PLUGIN);
                if ($packages_files_info !== false)
                {
                    foreach ($packages_files_info as $file_data)
                    {
                        if(!array_key_exists($file_data['file_name'],$exist_files_name))
                        {
                            $result['files'][]=$file_data;
                        }
                    }
                }
            }
            else if(WPVIVID_BACKUP_TYPE_UPLOADS_FILES==$backup_data['key'])
            {
                $packages_files_info = $this->get_packages_files_info(WPVIVID_BACKUP_TYPE_UPLOADS_FILES);
                if ($packages_files_info !== false)
                {
                    foreach ($packages_files_info as $file_data)
                    {
                        if(!array_key_exists($file_data['file_name'],$exist_files_name))
                        {
                            $result['files'][]=$file_data;
                        }
                    }
                }
            }
        }

        return $result;
    }

    public function get_backup_result()
    {
        $ret['result']=WPVIVID_SUCCESS;
        foreach ($this->task['options']['backup_options']['backup'] as $backup_data)
        {
            if($backup_data['result']!==false)
            {
                $ret=$backup_data['result'];
                if($ret['result']!==WPVIVID_SUCCESS)
                {
                    return $ret;
                }
            }
        }
        return $ret;
    }

    private function transfer_path($path)
    {
        $path = str_replace('\\','/',$path);
        $values = explode('/',$path);
        return implode(DIRECTORY_SEPARATOR,$values);
    }

    public function get_file_list($root,$exclude_regex,$include_regex,$exclude_file_size)
    {
        $files=array();
        $this->getFileLoop($files,$root,$exclude_regex,$include_regex,$exclude_file_size);
        return $files;
    }

    public function getFileLoop(&$files,$path,$exclude_regex=array(),$include_regex=array(),$exclude_file_size,$include_dir = true)
    {
        $count = 0;
        if(is_dir($path))
        {
            $handler = opendir($path);
            while (($filename = readdir($handler)) !== false) {
                if ($filename != "." && $filename != "..") {
                    $count++;

                    if (is_dir($path . DIRECTORY_SEPARATOR . $filename))
                    {
                        if ($this->regex_match($exclude_regex, $path . DIRECTORY_SEPARATOR . $filename, 0)) {
                            if ($this->regex_match($include_regex, $path . DIRECTORY_SEPARATOR . $filename, 1)) {
                                $this->getFileLoop($files, $path . DIRECTORY_SEPARATOR . $filename, $exclude_regex, $include_regex, $exclude_file_size, $include_dir);
                            }
                        }
                    } else {
                        if($this->regex_match($exclude_regex, $path . DIRECTORY_SEPARATOR . $filename, 0)){
                            if ($exclude_file_size == 0) {
                                $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                            } else {
                                if (filesize($path . DIRECTORY_SEPARATOR . $filename) < $exclude_file_size * 1024 * 1024) {
                                    $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                                }
                            }
                        }
                    }
                }
            }
            if($handler)
                @closedir($handler);
        }
        if($include_dir && $count == 0){
            $files[] = $path;
        }
    }

    private function regex_match($regex_array,$string,$mode)
    {
        if(empty($regex_array))
        {
            return true;
        }

        if($mode==0)
        {
            foreach ($regex_array as $regex)
            {
                if(preg_match($regex,$string))
                {
                    return false;
                }
            }

            return true;
        }

        if($mode==1)
        {
            foreach ($regex_array as $regex)
            {
                if(preg_match($regex,$string))
                {
                    return true;
                }
            }

            return false;
        }

        return true;
    }

    public function get_need_cleanup_files($all=false)
    {
        $files=array();
        if($this->task['options']['backup_options']['ismerge']==1)
        {
            foreach ($this->task['options']['backup_options']['backup'] as $backup_finished_data)
            {
                if($all===false)
                {
                    if(WPVIVID_BACKUP_TYPE_MERGE==$backup_finished_data['key'])
                        continue;
                }

                $files=array_merge($files,$this->get_backup_file($backup_finished_data['key']));
                add_filter('wpvivid_get_need_cleanup_files', array($this, 'wpvivid_get_need_cleanup_files'),10,2);
                $files=apply_filters('wpvivid_get_need_cleanup_files',$files,$backup_finished_data);
            }
        }

        return $files;
    }

    public function wpvivid_get_need_cleanup_files($files,$backup_finished_data)
    {
        if(WPVIVID_BACKUP_TYPE_PLUGIN==$backup_finished_data['key'])
        {
            $general_setting=WPvivid_Setting::get_setting(true, "");
            if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                    $packages_files_info = $this->get_packages_files_info(WPVIVID_BACKUP_TYPE_PLUGIN);
                    if ($packages_files_info !== false) {
                        foreach ($packages_files_info as $file_data) {
                            $path = WP_CONTENT_DIR . DIRECTORY_SEPARATOR . $this->task['options']['backup_options']['dir'] . DIRECTORY_SEPARATOR . $file_data['file_name'];
                            global $wpvivid_pulgin;
                            $wpvivid_pulgin->wpvivid_log->WriteLog('test:' . $path, 'notice');
                            if (!in_array($path, $files))
                                $files[] = $path;
                        }
                    }
                }
            }
        }
        else if(WPVIVID_BACKUP_TYPE_UPLOADS_FILES==$backup_finished_data['key'])
        {
            $general_setting=WPvivid_Setting::get_setting(true, "");
            if(isset($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']) && !empty($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload'])){
                if($general_setting['options']['wpvivid_compress_setting']['subpackage_plugin_upload']){
                    $packages_files_info = $this->get_packages_files_info(WPVIVID_BACKUP_TYPE_UPLOADS_FILES);
                    if ($packages_files_info !== false) {
                        foreach ($packages_files_info as $file_data) {
                            $path = WP_CONTENT_DIR . DIRECTORY_SEPARATOR . $this->task['options']['backup_options']['dir'] . DIRECTORY_SEPARATOR . $file_data['file_name'];
                            global $wpvivid_pulgin;
                            $wpvivid_pulgin->wpvivid_log->WriteLog('test:' . $path, 'notice');
                            if (!in_array($path, $files))
                                $files[] = $path;
                        }
                    }
                }
            }
        }
        return $files;
    }

    public function get_prefix()
    {
        return  $this->task['options']['backup_options']['prefix'];
    }

    public function set_file_and_db_info($db_size,$file_size)
    {
        $this->task['data']['backup']['db_size']=$db_size;
        $this->task['data']['backup']['files_size']=$file_size;

        WPvivid_Setting::update_task($this->task['id'],$this->task);
    }

    public function get_file_info()
    {
        $file_size['sum_size']=0;
        $file_size['sum_count']=0;

        $memory_limit = ini_get('memory_limit');
        $ret['memory_limit']=$memory_limit;
        $memory_limit = trim($memory_limit);
        $memory_limit_int = (int) $memory_limit;
        $last = strtolower(substr($memory_limit, -1));

        if($last == 'g')
            $memory_limit_int = $memory_limit_int*1024*1024*1024;
        if($last == 'm')
            $memory_limit_int = $memory_limit_int*1024*1024;
        if($last == 'k')
            $memory_limit_int = $memory_limit_int*1024;

        $files=array();

        $exclude_size= $this->task['options']['backup_options']['compress']['exclude_file_size'];

        foreach ($this->task['options']['backup_options']['backup'] as $backup_data)
        {
            if(!isset($backup_data['dump_db']))
            {
                $backup_files=$this->get_need_backup_files($backup_data);
                $files=array_merge($backup_files,$files);
            }
        }

        foreach ($files as $file)
        {
            $size=0;
            $_file_size=filesize($file);
            if($_file_size>($memory_limit_int*0.9))
            {
                $ret['alter_big_file']=true;
                $ret['alter_files']=true;
            }
            $size+=$_file_size;
            $file_size['sum_count']++;
            $file_size['sum_size']+=$size;
        }
        return $file_size;
    }

    public function is_cancel_file_exist()
    {
        $file_name=$this->task['options']['file_prefix'];

        $file=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->task['options']['backup_options']['dir'].DIRECTORY_SEPARATOR.$file_name.'_cancel';

        if(file_exists($file))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    public function update_status($status)
    {
        $this->task['status']['str']=$status;
        WPvivid_Setting::update_task($this->task['id'],$this->task);
    }

    public function get_backup_task_info($task_id){
        $list_tasks['status']=WPvivid_taskmanager::get_backup_tasks_status($task_id);
        $list_tasks['is_canceled']=WPvivid_taskmanager::is_task_canceled($task_id);
        $list_tasks['size']=WPvivid_taskmanager::get_backup_size($task_id);
        $list_tasks['data']=WPvivid_taskmanager::get_backup_tasks_progress($task_id);
        //
        $list_tasks['task_info']['need_next_schedule']=false;
        if($list_tasks['status']['str']=='running'||$list_tasks['status']['str']=='no_responds')
        {
            if($list_tasks['data']['running_time']>180) {
                $list_tasks['task_info']['need_next_schedule'] = true;
            }
            else{
                $list_tasks['task_info']['need_next_schedule'] = false;
            }
        }
        //
        $general_setting=WPvivid_Setting::get_setting(true, "");
        if($general_setting['options']['wpvivid_common_setting']['estimate_backup'] == 0){
            $list_tasks['task_info']['display_estimate_backup'] = 'display: none';
        }
        else{
            $list_tasks['task_info']['display_estimate_backup'] = '';
        }
        //
        $list_tasks['task_info']['backup_percent']=$list_tasks['data']['progress'].'%';
        //
        if($list_tasks['size']['db_size'] == false){
            $list_tasks['task_info']['db_size']=0;
        }
        else{
            $list_tasks['task_info']['db_size']=$list_tasks['size']['db_size'];
        }
        if($list_tasks['size']['files_size'] == false){
            $list_tasks['task_info']['file_size']=0;
        }
        else{
            $list_tasks['task_info']['file_size']=$list_tasks['size']['files_size']['sum'];
        }
        //
        $list_tasks['task_info']['descript']='';
        $list_tasks['task_info']['css_btn_cancel']='pointer-events: auto; opacity: 1;';
        $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
        $list_tasks['task_info']['total'] = 'N/A';
        $list_tasks['task_info']['upload'] = 'N/A';
        $list_tasks['task_info']['speed'] = 'N/A';
        $list_tasks['task_info']['network_connection'] = 'N/A';

        $list_tasks['task_info']['need_update_last_task']=false;
        if($list_tasks['status']['str']=='ready')
        {
            $list_tasks['task_info']['descript']=__('Ready to backup. Progress: 0%, running time: 0second.','wpvivid');
            $list_tasks['task_info']['css_btn_cancel']='pointer-events: none; opacity: 0.4;';
            $list_tasks['task_info']['css_btn_log']='pointer-events: none; opacity: 0.4;';
        }
        else if($list_tasks['status']['str']=='running')
        {
            if($list_tasks['is_canceled'] == false)
            {
                if($list_tasks['data']['type'] == 'upload')
                {
                    if(isset($list_tasks['data']['upload_data']) && !empty($list_tasks['data']['upload_data'])) {
                        $descript = $list_tasks['data']['upload_data']['descript'];
                        $offset = $list_tasks['data']['upload_data']['offset'];
                        $current_size = $list_tasks['data']['upload_data']['current_size'];
                        $last_time = $list_tasks['data']['upload_data']['last_time'];
                        $last_size = $list_tasks['data']['upload_data']['last_size'];
                        $speed = ($offset - $last_size) / (time() - $last_time);
                        $speed /= 1000;
                        $speed = round($speed, 2);
                        $speed .= 'kb/s';
                        if(!empty($current_size)) {
                            $list_tasks['task_info']['total'] = size_format($current_size,2);
                        }
                        if(!empty($offset)) {
                            $list_tasks['task_info']['upload'] = size_format($offset, 2);
                        }
                    }
                    else{
                        $descript = 'Strat uploading.';
                        $speed = '0kb/s';
                        $list_tasks['task_info']['total'] = 'N/A';
                        $list_tasks['task_info']['upload'] = 'N/A';
                    }

                    $list_tasks['task_info']['speed'] = $speed;
                    $list_tasks['task_info']['descript'] = $descript.__(' Progress: ', 'wpvivid') . $list_tasks['task_info']['backup_percent'] . __(', running time: ', 'wpvivid') . $list_tasks['data']['running_time'];

                    $time_spend=time()-$list_tasks['status']['run_time'];
                    if($time_spend>30)
                    {
                        $list_tasks['task_info']['network_connection']='Retrying';
                    }
                    else
                    {
                        $list_tasks['task_info']['network_connection']='OK';
                    }
                }
                else {
                    $list_tasks['task_info']['descript'] = $list_tasks['data']['descript'] . __(' Progress: ', 'wpvivid') . $list_tasks['task_info']['backup_percent'] . __(', running time: ', 'wpvivid') . $list_tasks['data']['running_time'];
                }
                $list_tasks['task_info']['css_btn_cancel']='pointer-events: auto; opacity: 1;';
                $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
            }
            else{
                $list_tasks['task_info']['descript']=__('The backup will be canceled after backing up the current chunk ends.','wpvivid');
                $list_tasks['task_info']['css_btn_cancel']='pointer-events: none; opacity: 0.4;';
                $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
            }
        }
        else if($list_tasks['status']['str']=='wait_resume'){
            $list_tasks['task_info']['descript']='Task '.$task_id.' timed out, backup task will retry in '.$list_tasks['data']['next_resume_time'].' seconds, retry times: '.$list_tasks['status']['resume_count'].'.';
            $list_tasks['task_info']['css_btn_cancel']='pointer-events: auto; opacity: 1;';
            $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
        }
        else if($list_tasks['status']['str']=='no_responds'){
            if($list_tasks['is_canceled'] == false){
                $list_tasks['task_info']['descript']='Task , '.$list_tasks['data']['doing'].' is not responding. Progress: '.$list_tasks['task_info']['backup_percent'].', running time: '.$list_tasks['data']['running_time'];
                $list_tasks['task_info']['css_btn_cancel']='pointer-events: auto; opacity: 1;';
                $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
            }
            else{
                $list_tasks['task_info']['descript']=__('The backup will be canceled after backing up the current chunk ends.','wpvivid');
                $list_tasks['task_info']['css_btn_cancel']='pointer-events: none; opacity: 0.4;';
                $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
            }
        }
        else if($list_tasks['status']['str']=='completed'){
            $list_tasks['task_info']['descript']='Task '.$task_id.' completed.';
            $list_tasks['task_info']['css_btn_cancel']='pointer-events: auto; opacity: 1;';
            $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
            $list_tasks['task_info']['need_update_last_task']=true;
        }
        else if($list_tasks['status']['str']=='error'){
            $list_tasks['task_info']['descript']='Backup error: '.$list_tasks['status']['error'];
            $list_tasks['task_info']['css_btn_cancel']='pointer-events: auto; opacity: 1;';
            $list_tasks['task_info']['css_btn_log']='pointer-events: auto; opacity: 1;';
            $list_tasks['task_info']['need_update_last_task']=true;
        }

        return $list_tasks;
    }

    public function get_packages_info($key)
    {
        if(isset($this->task['data']['backup']['sub_job'][$key]))
        {
            if(empty($this->task['data']['backup']['sub_job'][$key]['job_data']))
            {
                return false;
            }
            else
            {
                $packages=array();
                foreach ($this->task['data']['backup']['sub_job'][$key]['job_data'] as $key=>$package)
                {
                    if($key=='files')
                    {
                        continue;
                    }

                    $packages[]=$package;
                }
                if(empty($packages))
                {
                    return false;
                }
                else
                {
                    return $packages;
                }
            }
        }
        else
        {
            return false;
        }
    }

    public function get_packages_files_info($key)
    {
        if(isset($this->task['data']['backup']['sub_job'][$key]))
        {
            if(empty($this->task['data']['backup']['sub_job'][$key]['job_data']))
            {
                return false;
            }
            else
            {
                if(isset($this->task['data']['backup']['sub_job'][$key]['job_data']['files']))
                {
                    return $this->task['data']['backup']['sub_job'][$key]['job_data']['files'];
                }
                else
                {
                    return false;
                }
            }
        }
        else
        {
            return false;
        }
    }

    public function set_packages_info($key,$packages)
    {
        $this->task=WPvivid_taskmanager::get_task($this->task['id']);
        $job_data=array();
        foreach ($packages as $package)
        {
            $package['backup']=false;
            $job_data[basename($package['path'])]=$package;
        }
        $this->task['data']['backup']['sub_job'][$key]['job_data']=$job_data;
        WPvivid_Setting::update_task($this->task['id'],$this->task);
        return $job_data;
    }

    public function update_packages_info($key,$package,$file_data=false)
    {
        $this->task=WPvivid_taskmanager::get_task($this->task['id']);
        $this->task['data']['backup']['sub_job'][$key]['job_data'][basename($package['path'])]=$package;
        if($file_data!==false)
        {
            $this->task['data']['backup']['sub_job'][$key]['job_data']['files'][]=$file_data;
        }
        WPvivid_Setting::update_task($this->task['id'],$this->task);
    }

    public function update_sub_task_progress($key,$finished,$progress)
    {
        $this->task=WPvivid_taskmanager::get_task($this->task['id']);
        $this->task['status']['run_time']=time();
        $this->task['status']['str']='running';
        $this->task['data']['doing']='backup';
        $sub_job_name=$key;
        $this->task['data']['backup']['doing']=$key;
        $this->task['data']['backup']['sub_job'][$sub_job_name]['finished']=$finished;
        $this->task['data']['backup']['sub_job'][$sub_job_name]['progress']=$progress;
        if(!isset( $this->task['data']['backup']['sub_job'][$sub_job_name]['job_data']))
        {
            $this->task['data']['backup']['sub_job'][$sub_job_name]['job_data']=array();
        }
        WPvivid_Setting::update_task($this->task['id'],$this->task);
    }

    public function get_need_backup_folder($backup_data)
    {
        if(isset($backup_data['files'])&&empty($backup_data['files']))
        {
            return $backup_data['files'];
        }
        else
        {
            return $this->get_folder_list($backup_data['files_root'],$backup_data['exclude_regex'],$backup_data['include_regex'],$this->task['options']['backup_options']['compress']['exclude_file_size']);
        }
    }

    public function get_folder_list($root,$exclude_regex,$include_regex,$exclude_file_size)
    {
        $files=array();
        $this->getFolder($files,$root,$exclude_regex,$include_regex,$exclude_file_size);
        return $files;
    }

    public function getFolder(&$files,$path,$exclude_regex=array(),$include_regex=array(),$exclude_file_size=array(),$include_dir = true)
    {
        $count = 0;
        if(is_dir($path))
        {
            $handler = opendir($path);
            while (($filename = readdir($handler)) !== false)
            {
                if ($filename != "." && $filename != "..")
                {
                    $count++;

                    if (is_dir($path . DIRECTORY_SEPARATOR . $filename))
                    {
                        if ($this->regex_match($exclude_regex, $path . DIRECTORY_SEPARATOR . $filename, 0))
                        {
                            if ($this->regex_match($include_regex, $path . DIRECTORY_SEPARATOR . $filename, 1))
                            {
                                $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                            }
                        }
                    } else {
                        if($this->regex_match($exclude_regex, $path . DIRECTORY_SEPARATOR . $filename, 0)){
                            if ($exclude_file_size == 0)
                            {
                                $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                            } else {
                                if (filesize($path . DIRECTORY_SEPARATOR . $filename) < $exclude_file_size * 1024 * 1024)
                                {
                                    $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                                }
                            }
                        }
                    }
                }
            }
            if($handler)
                @closedir($handler);
        }
        if($include_dir && $count == 0)
        {
            $files[] = $path;
        }
    }

    public function get_need_uploads_backup_folder($backup_data)
    {
        if(isset($backup_data['files'])&&empty($backup_data['files']))
        {
            return $backup_data['files'];
        }
        else
        {
            return $this->get_uploads_folder_list($backup_data['files_root'],$backup_data['exclude_regex'],$backup_data['include_regex'],$this->task['options']['backup_options']['compress']['exclude_file_size']);
        }
    }

    public function get_uploads_folder_list($root,$exclude_regex,$include_regex,$exclude_file_size)
    {
        $files=array();
        $this->getUploadsFolder($files,$root,$exclude_regex,$include_regex,$exclude_file_size);
        return $files;
    }

    public function getUploadsFolder(&$files,$path,$exclude_regex=array(),$include_regex=array(),$exclude_file_size=array(),$include_dir = true,$subdir=false)
    {
        $count = 0;
        if(is_dir($path))
        {
            $handler = opendir($path);
            while (($filename = readdir($handler)) !== false)
            {
                if ($filename != "." && $filename != "..")
                {
                    $count++;

                    if (is_dir($path . DIRECTORY_SEPARATOR . $filename))
                    {
                        if ($this->regex_match($exclude_regex, $path . DIRECTORY_SEPARATOR . $filename, 0))
                        {
                            if ($this->regex_match($include_regex, $path . DIRECTORY_SEPARATOR . $filename, 1))
                            {
                                if($subdir)
                                    $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                                else
                                    $this->getUploadsFolder($files,$path . DIRECTORY_SEPARATOR . $filename,$exclude_regex,$include_regex,$exclude_file_size,$include_dir,true);
                            }
                        }
                    } else {
                        if($this->regex_match($exclude_regex, $path . DIRECTORY_SEPARATOR . $filename, 0))
                        {
                            if ($exclude_file_size == 0)
                            {
                                //if($subdir)
                                //    $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                            } else {
                                if (filesize($path . DIRECTORY_SEPARATOR . $filename) < $exclude_file_size * 1024 * 1024)
                                {
                                    //if($subdir)
                                    //    $files[] = $path . DIRECTORY_SEPARATOR . $filename;
                                }
                            }
                        }
                    }
                }
            }
            if($handler)
                @closedir($handler);
        }
        if($include_dir && $count == 0)
        {
            if($subdir)
                $files[] = $path;
        }
    }
}

class WPvivid_Backup_Item
{
    private $config;

    public function __construct($options)
    {
        $this->config=$options;
    }

    public function get_backup_type()
    {
        return $this->config['type'];
    }

    public function get_files($has_dir=true)
    {
        $files=array();
        if(isset($this->config['backup']['files']))
        {
            //file_name
            foreach ($this->config['backup']['files'] as $file)
            {
                if($has_dir)
                    $files[]=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file['file_name'];
                else
                    $files[]=$file['file_name'];
            }
        }
        else{
            if(isset($this->config['backup']['data']['meta']['files']))
            {
                foreach ($this->config['backup']['data']['meta']['files'] as $file)
                {
                    if($has_dir)
                        $files[]=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file['file_name'];
                    else
                        $files[]=$file['file_name'];
                }
            }
        }
        return $files;
    }

    public function is_lock()
    {
        if(isset($this->config['lock']))
        {
            return $this->config['lock'];
        }
        else{
            return false;
        }
    }

    public function check_backup_files()
    {
        global $wpvivid_pulgin;
        $ret['result']=WPVIVID_FAILED;
        $ret['error']='Unknown error.';

        $b_has_data=false;
        $tmp_data=array();
        if(isset($this->config['backup']['files']))
        {
            $b_has_data = true;
            $tmp_data = $this->config['backup']['files'];
        }
        else if(isset($this->config['backup']['data']['meta']['files'])){
            $b_has_data = true;
            $tmp_data = $this->config['backup']['data']['meta']['files'];
        }

        if($b_has_data){
            $b_need_download=false;
            $b_not_found=false;
            $b_test=false;
            foreach ($tmp_data as $file)
            {
                $need_download=false;
                $path=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file['file_name'];
                if(file_exists($path))
                {
                    if(filesize($path) === $file['size']){
                        if($wpvivid_pulgin->wpvivid_check_zip_valid()) {
                            $res = TRUE;
                        }
                        else{
                            $res = FALSE;
                        }
                    }
                    else{
                        $res = FALSE;
                    }
                    if ($res !== TRUE)
                    {
                        $need_download=true;
                    }
                }
                else
                {
                    $b_test=true;
                    $test_str=$path;
                    $need_download=true;
                }

                if($need_download)
                {
                    if(empty($this->config['remote']))
                    {
                        $b_not_found=true;
                        $ret['files'][$file['file_name']]['status']='file_not_found';
                        $ret['files'][$file['file_name']]['size']=$file['size'];
                        $ret['files'][$file['file_name']]['md5']=$file['md5'];
                    }
                    else
                    {
                        $b_need_download=true;
                        WPvivid_taskmanager::delete_download_task_v2($file['file_name']);
                        $ret['files'][$file['file_name']]['status']='need_download';
                        $ret['files'][$file['file_name']]['size']=$file['size'];
                        $ret['files'][$file['file_name']]['md5']=$file['md5'];
                    }
                }
            }

            if($b_not_found)
            {
                $ret['result']=WPVIVID_FAILED;
                if($b_test)
                    $ret['error']='Backup files doesn\'t exist. Restore failed.'.$test_str;
                else
                    $ret['error']='Backup doesn\'t exist in both web server and remote storage. Restore failed.';
            }
            else if($b_need_download)
            {
                $ret['result']='need_download';
            }
            else
            {
                $ret['result']=WPVIVID_SUCCESS;
            }
        }

        return $ret;
    }

    public function check_migrate_file()
    {
        if(isset($this->config['backup']['files']))
        {
            $tmp_data = $this->config['backup']['files'];
            $zip=new WPvivid_ZipClass();

            foreach ($tmp_data as $file)
            {
                $path=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file['file_name'];
                if(file_exists($path))
                {
                    $json=$zip->get_json_data($path);
                    $json=json_decode($json,1);
                    if(!is_null($json))
                    {
                        if(isset($json['home_url'])&&home_url()!=$json['home_url'])
                        {
                            return 1;
                        }
                    }
                }
            }

            return 0;
        }
        else
        {
            return 0;
        }

    }

    public function get_local_path()
    {
        return $this->config['local']['path'];
    }

    public function get_remote()
    {
        $remote_option=array_shift($this->config['remote']);

        if(is_null($remote_option))
        {
            return false;
        }
        else
        {
            return $remote_option;
        }
    }

    public function get_backup_packages()
    {
        $packages=array();
        $index=0;

        if(isset($this->config['backup']['files']))
        {
            $file_added=array();
            //file_name
            foreach ($this->config['backup']['files'] as $file)
            {
                if(isset($file_added[$file['file_name']]))
                {
                    continue;
                }

                if(strpos($file['file_name'],'part')!==false)
                {
                    $this->get_all_part_files($file['file_name'],$this->config['backup']['files'],$packages[$index],$file_added);
                }
                else
                {
                    $packages[$index]['files'][]=$file['file_name'];
                    $file_added[$file['file_name']]=1;
                }
                $index++;
            }

            $file_added=array();
            $child_packages=array();
            foreach ($packages as $package)
            {
                $files=array();

                foreach ($package['files'] as $package_files)
                {
                    $files=array_merge($files,$this->get_child_files($package_files));
                }

                if(empty($files))
                {
                    continue;
                }

                foreach ($files as $file)
                {
                    if(isset($file_added[$file['file_name']]))
                    {
                        continue;
                    }

                    if(strpos($file['file_name'],'part')!==false)
                    {
                        $this->get_all_part_files($file['file_name'],$files,$child_packages[$index],$file_added);
                    }
                    else
                    {
                        $child_packages[$index]['files'][]=$file['file_name'];
                        $file_added[$file['file_name']]=1;
                    }
                    $index++;
                }
            }

            $packages=array_merge($packages,$child_packages);
        }
        else if(isset($this->config['backup']['data']))
        {
            if(isset($this->config['backup']['ismerge'])&&$this->config['backup']['ismerge']==1)
            {
                $packages[$index]['option']['has_child']=1;
                $packages[$index]['option']['root']='wp-content';
                foreach ($this->config['backup']['data']['meta']['files'] as $file)
                {
                    $packages[$index]['files'][]=$file['file_name'];
                }
                $index++;
            }

            foreach ($this->config['backup']['data']['type'] as $type)
            {
                if($type['type_name']=='backup_db')
                {
                    $packages[$index]['option']['dump_db']=1;
                    $packages[$index]['option']['root']='wp-content\\'.$this->config['local']['path'];
                }
                else if($type['type_name']=='backup_themes')
                {
                    $packages[$index]['option']['root']='wp-content';
                }
                else if($type['type_name']=='backup_plugin')
                {
                    $packages[$index]['option']['root']='wp-content';
                }
                else if($type['type_name']=='backup_uploads')
                {
                    $packages[$index]['option']['root']='wp-content';
                }
                else if($type['type_name']=='backup_content')
                {
                    $packages[$index]['option']['root']='';
                }
                else if($type['type_name']=='backup_core')
                {
                    $packages[$index]['option']['root']='';
                    $packages[$index]['option']['include_path'][]='wp-includes';
                    $packages[$index]['option']['include_path'][]='wp-admin';
                }

                foreach ($type['files'] as $file)
                {
                    $packages[$index]['files'][]=$file['file_name'];
                }
                $index++;
            }
        }

        return $packages;
    }

    public function get_all_part_files($file_name,$files,&$package,&$file_added)
    {
        $i=strpos($file_name,'part');

        $file_prefix=substr($file_name,0,$i);

        foreach ($files as $file)
        {
            if(isset($file_added[$file['file_name']]))
            {
                continue;
            }

            if (strpos($file['file_name'], $file_prefix) !== false)
            {
                $package['files'][]=$file['file_name'];
                $file_added[$file['file_name']]=1;
            }
        }
    }

    public function get_child_files($file_name)
    {
        $zip=new WPvivid_ZipClass();

        $path=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file_name;

        $json=$zip->get_json_data($path);

        $json=json_decode($json,1);

        $files=array();

        if(isset($json['has_child']))
        {
            $files=$zip->list_file($path);
        }

        return $files;
    }

    public function get_file_info($file_name)
    {
        $zip=new WPvivid_ZipClass();

        $path=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file_name;

        $json=$zip->get_json_data($path);

        $json=json_decode($json,1);
        if(is_null($json))
        {
            return false;
        }
        else
        {
            return $json;
        }
    }

    static public function get_backup_file_info($file_name)
    {
        $zip=new WPvivid_ZipClass();
        $json=$zip->get_json_data($file_name);
        $json=json_decode($json,1);
        if(is_null($json))
        {
            return false;
        }
        else
        {
            return $json;
        }
    }

    public function get_sql_file($file_name)
    {
        $zip=new WPvivid_ZipClass();
        $path=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file_name;
        $files=$zip->list_file($path);
        return $files[0]['file_name'];
    }

    public static function get_backup_files($backup){
        $files=array();
        if(isset($backup['backup']['files'])){
            $files=$backup['backup']['files'];
        }
        else{
            if(isset($backup['backup']['ismerge'])) {
                if ($backup['backup']['ismerge'] == 1) {
                    if(isset($backup['backup']['data']['meta']['files'])){
                        $files=$backup['backup']['data']['meta']['files'];
                    }
                }
            }
        }
        return $files;
    }

    public function get_download_backup_files($backup_id){
        $ret['result']=WPVIVID_FAILED;
        $data=array();
        $backup=WPvivid_Backuplist::get_backuplist_by_key($backup_id);
        if(!$backup)
        {
            $ret['error']='Backup id not found.';
            return $ret;
        }

        $files=array();
        $files = self::get_backup_files($backup);
        if(empty($files)){
            $ret['error']='Failed to get backup files.';
        }
        else{
            $ret['result']=WPVIVID_SUCCESS;
            $ret['files']=$files;
        }
        return $ret;
    }

    public function get_download_progress($backup_id, $files){
        global $wpvivid_pulgin;
        $b_need_download=false;
        $b_not_found=false;
        $file_count=0;
        $file_part_num=1;
        $check_type='';
        foreach ($files as $file)
        {
            $need_download=false;
            $path=WP_CONTENT_DIR.DIRECTORY_SEPARATOR.$this->config['local']['path'].DIRECTORY_SEPARATOR.$file['file_name'];
            if(file_exists($path)) {
                if(filesize($path) === $file['size']){
                    if($wpvivid_pulgin->wpvivid_check_zip_valid()) {
                        $res = TRUE;
                    }
                    else{
                        $res = FALSE;
                    }
                }
                else{
                    $res = FALSE;
                }
                if ($res !== TRUE)
                {
                    $need_download=true;
                }
            }
            else {
                $need_download=true;
            }
            if($file_part_num < 10){
                $format_part=sprintf("%02d", $file_part_num);
            }
            else{
                $format_part=$file_part_num;
            }
            if($need_download) {
                if(empty($this->config['remote'])) {
                    $b_not_found=true;
                    $ret['result'] = WPVIVID_SUCCESS;
                    $ret['files'][$file['file_name']]['status']='file_not_found';
                }
                else{
                    $task = WPvivid_taskmanager::get_download_task_v2($file['file_name']);
                    if ($task === false) {
                        $ret['result'] = WPVIVID_SUCCESS;
                        $ret['files'][$file['file_name']]['status']='need_download';
                        $ret['files'][$file['file_name']]['html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px;">
                                                                  <span>Part'.$format_part.'</span></br>
                                                                  <span id=\''.$backup_id.'-text-part-'.$file_part_num.'\'><a onclick="wpvivid_prepare_download(\''.$file_part_num.'\', \''.$backup_id.'\', \''.$file['file_name'].'\');" style="cursor: pointer;">Prepare to Download</a></span></br>
                                                                  <div style="width:100%;height:5px; background-color:#dcdcdc;"><div id=\''.$backup_id.'-progress-part-'.$file_part_num.'\' style="background-color:#0085ba; float:left;width:0;height:5px;"></div></div>
                                                                  <span>size:</span><span>'.$wpvivid_pulgin->formatBytes($file['size']).'</span>
                                                                  </div>';
                    } else {
                        $ret['result'] = WPVIVID_SUCCESS;
                        if($task['status'] === 'running'){
                            $ret['files'][$file['file_name']]['status'] = 'running';
                            $ret['files'][$file['file_name']]['html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px;">
                                                                            <span>Part'.$format_part.'</span></br>
                                                                            <span id=\''.$backup_id.'-text-part-'.$file_part_num.'\'><a >Retriving(remote storage to web server)</a></span></br>
                                                                            <div style="width:100%;height:5px; background-color:#dcdcdc;"><div id=\''.$backup_id.'-progress-part-'.$file_part_num.'\' style="background-color:#0085ba; float:left;width:'.$task['progress_text'].'%;height:5px;"></div></div>
                                                                            <span>size:</span><span>'.$wpvivid_pulgin->formatBytes($file['size']).'</span>
                                                                            </div>';
                            $ret['files'][$file['file_name']]['progress_text']=$task['progress_text'];
                        }
                        elseif($task['status'] === 'timeout'){
                            $ret['files'][$file['file_name']]['status']='timeout';
                            $ret['files'][$file['file_name']]['html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px;">
                                                                            <span>Part'.$format_part.'</span></br>
                                                                            <span id=\''.$backup_id.'-text-part-'.$file_part_num.'\'><a onclick="wpvivid_prepare_download(\''.$file_part_num.'\', \''.$backup_id.'\', \''.$file['file_name'].'\');" style="cursor: pointer;">Prepare to Download</a></span></br>
                                                                            <div style="width:100%;height:5px; background-color:#dcdcdc;"><div id=\''.$backup_id.'-progress-part-'.$file_part_num.'\' style="background-color:#0085ba; float:left;width:'.$task['progress_text'].'%;height:5px;"></div></div>
                                                                            <span>size:</span><span>'.$wpvivid_pulgin->formatBytes($file['size']).'</span>
                                                                            </div>';
                            $ret['files'][$file['file_name']]['progress_text']=$task['progress_text'];
                            WPvivid_taskmanager::delete_download_task_v2($file['file_name']);
                        }
                        elseif($task['status'] === 'completed'){
                            $ret['files'][$file['file_name']]['status']='completed';
                            $ret['files'][$file['file_name']]['html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px;">
                                                                 <span>Part'.$format_part.'</span></br>
                                                                 <span id=\''.$backup_id.'-text-part-'.$file_part_num.'\'><a onclick="wpvivid_download(\''.$backup_id.'\', \''.$check_type.'\', \''.$file['file_name'].'\');" style="cursor: pointer;">Download</a></span></br>
                                                                 <div style="width:100%;height:5px; background-color:#dcdcdc;"><div id=\''.$backup_id.'-progress-part-'.$file_part_num.'\' style="background-color:#0085ba; float:left;width:100%;height:5px;"></div></div>
                                                                 <span>size:</span><span>'.$wpvivid_pulgin->formatBytes($file['size']).'</span>
                                                                 </div>';
                            WPvivid_taskmanager::delete_download_task_v2($file['file_name']);
                        }
                        elseif($task['status'] === 'error'){
                            $ret['files'][$file['file_name']]['status']='error';
                            $ret['files'][$file['file_name']]['html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px;">
                                                                        <span>Part'.$format_part.'</span></br>
                                                                        <span id=\''.$backup_id.'-text-part-'.$file_part_num.'\'><a onclick="wpvivid_prepare_download(\''.$file_part_num.'\', \''.$backup_id.'\', \''.$file['file_name'].'\');" style="cursor: pointer;">Prepare to Download</a></span></br>
                                                                        <div style="width:100%;height:5px; background-color:#dcdcdc;"><div id=\''.$backup_id.'-progress-part-'.$file_part_num.'\' style="background-color:#0085ba; float:left;width:0;height:5px;"></div></div>
                                                                        <span>size:</span><span>'.$wpvivid_pulgin->formatBytes($file['size']).'</span>
                                                                        </div>';
                            $ret['files'][$file['file_name']]['error'] = $task['error'];
                            WPvivid_taskmanager::delete_download_task_v2($file['file_name']);
                        }
                    }
                }
            }
            else{
                $ret['result'] = WPVIVID_SUCCESS;
                if(WPvivid_taskmanager::get_download_task_v2($file['file_name']))
                    WPvivid_taskmanager::delete_download_task_v2($file['file_name']);
                $ret['files'][$file['file_name']]['status']='completed';
                $ret['files'][$file['file_name']]['download_path']=$path;
                $ret['files'][$file['file_name']]['html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px;">
                                                                 <span>Part'.$format_part.'</span></br>
                                                                 <span id=\''.$backup_id.'-text-part-'.$file_part_num.'\'><a onclick="wpvivid_download(\''.$backup_id.'\', \''.$check_type.'\', \''.$file['file_name'].'\');" style="cursor: pointer;">Download</a></span></br>
                                                                 <div style="width:100%;height:5px; background-color:#dcdcdc;"><div id=\''.$backup_id.'-progress-part-'.$file_part_num.'\' style="background-color:#0085ba; float:left;width:100%;height:5px;"></div></div>
                                                                 <span>size:</span><span>'.$wpvivid_pulgin->formatBytes($file['size']).'</span>
                                                                 </div>';
            }
            $ret['files'][$file['file_name']]['size']=$wpvivid_pulgin->formatBytes($file['size']);
            $file_count++;
            $file_part_num++;
        }
        if ($file_count % 2 != 0) {
            $file_count++;
            if($file_count < 10){
                $format_part=sprintf("%02d", $file_count);
            }
            else{
                $format_part=$file_count;
            }
            $ret['place_html']='<div style="float:left;margin:10px 10px 10px 0;text-align:center; width:180px; color:#cccccc;">
                                   <span>Part'.$format_part.'</span></br>
                                   <span>Download</span></br>
                                   <div style="width:100%;height:5px; background-color:#dcdcdc;"><div style="background-color:#0085ba; float:left;width:0;height:5px;"></div></div>
                                   <span>size:</span><span>0</span>
                                   </div>';
        }
        else{
            $ret['place_html']='';
        }
        return $ret;
    }

    public function update_download_page($backup_id){
        $ret=$this->get_download_backup_files($backup_id);
        if($ret['result']==WPVIVID_SUCCESS){
            $ret=$this->get_download_progress($backup_id, $ret['files']);
            WPvivid_taskmanager::update_download_cache($backup_id,$ret);
        }
        return $ret;
    }
}

class WPvivid_Backup
{
    public $task;
    //public $config;

    public function __construct($task_id=false,$task=false)
    {
        if($task_id!==false)
        {
            $this->task=new WPvivid_Backup_Task($task_id);
        }
        else if($task!==false)
        {
            $this->task=new WPvivid_Backup_Task(false,$task);
        }
        else
        {
            $this->task=new WPvivid_Backup_Task();
        }
        //$this->config=$config;
    }

	public function init_options($task_id)
    {
        $this->task=new WPvivid_Backup_Task($task_id);
    }

	public function backup($task_id)
    {
        $this->init_options($task_id);
        $next_backup=$this->task->get_need_backup();

        while($next_backup!==false)
        {
            global $wpvivid_pulgin;
            $wpvivid_pulgin->set_time_limit($this->task->get_id());
            $this->task->update_sub_task_progress($next_backup['key'],0,'Start backing up '.$next_backup['key'].'.');
            $wpvivid_pulgin->wpvivid_log->WriteLog('Prepare to backup '.$next_backup['key'].' files.','notice');
            if(isset($next_backup['files'])) {
                $wpvivid_pulgin->wpvivid_log->WriteLog('File number: ' . sizeof($next_backup['files']), 'notice');
            }
            $result = $this->_backup($next_backup);
            $wpvivid_pulgin->wpvivid_log->WriteLog('Backing up '.$next_backup['key'].' completed.','notice');
            $this->task->update_sub_task_progress($next_backup['key'],1,'Backing up '.$next_backup['key'].' finished.');
            $this->task->update_backup_result($next_backup,$result);
            $wpvivid_pulgin->check_cancel_backup($task_id);
            unset($next_backup);
            $next_backup=$this->task->get_need_backup();
        }

        $this->cleanup();

        $ret=$this->task->get_backup_result();
        return $ret;
	}

    private function _backup($data)
    {
        global $wpvivid_pulgin;
        $result['result']=WPVIVID_FAILED;
        $result['error']='test error';
        if(isset($data['dump_db']))
        {
            include_once WPVIVID_PLUGIN_DIR .'/includes/class-wpvivid-backup-database.php';
            $wpvivid_pulgin->wpvivid_log->WriteLog('Start exporting database.','notice');
            $backup_database = new WPvivid_Backup_Database();
            $result = $backup_database -> backup_database($data,$this->task->get_id());
            $wpvivid_pulgin->wpvivid_log->WriteLog('Exporting database finished.','notice');
            if($result['result']==WPVIVID_SUCCESS)
            {
                $data['files']=$result['files'];
            }
            else
            {
                return $result;
            }
        }

        include_once WPVIVID_PLUGIN_DIR . '/includes/class-wpvivid-zipclass.php';

        $zip = new WPvivid_ZipClass();
        if(is_array($zip->last_error))
        {
            return $zip->last_error;
        }

        if(isset($data['resume_packages']))
        {
            global $wpvivid_pulgin;
            $wpvivid_pulgin->wpvivid_log->WriteLog('Start compressing '.$data['key'],'notice');

            if($data['key']==WPVIVID_BACKUP_TYPE_PLUGIN)
            {
                $ret =$zip->get_plugin_packages($data);
            }
            else if($data['key']==WPVIVID_BACKUP_TYPE_UPLOADS_FILES)
            {
                $ret =$zip->get_upload_packages($data);
            }
            else
            {
                $ret =$zip->get_packages($data);
            }

            $packages=$this->task->get_packages_info($data['key']);
            if($packages===false)
            {
                $packages=$this->task->set_packages_info($data['key'],$ret['packages']);
            }

            define(PCLZIP_TEMPORARY_DIR,$ret['temp_dir']);

            $result['result']=WPVIVID_SUCCESS;
            $result['files']=array();

            foreach ($packages as $package)
            {
                if(!empty($package['files'])&&$package['backup']==false)
                {
                    $zip_ret=$zip->_zip($package['path'],$package['files'], $data,$package['json']);
                    if($zip_ret['result']==WPVIVID_SUCCESS)
                    {
                        $result['files'][] = $zip_ret['file_data'];
                        $package['backup']=true;
                        $this->task->update_packages_info($data['key'],$package,$zip_ret['file_data']);
                    }
                    else
                    {
                        $result=$zip_ret;
                        break;
                    }
                }else {
                    continue;
                }
            }
            $wpvivid_pulgin->wpvivid_log->WriteLog('Compressing '.$data['key'].' completed','notice');
            return $result;
        }
        else
        {
            $result =$zip->compress($data);

            if(isset($data['dump_db']))
            {
                foreach ($data['files'] as $sql_file)
                {
                    @unlink($sql_file);
                }
            }
        }

        return $result;

        /*
        if(isset($data['has_child']))
        {
            global $wpvivid_pulgin;
            $wpvivid_pulgin->wpvivid_log->WriteLog('Start compressing '.$data['key'],'notice');
            $wpvivid_pulgin->wpvivid_log->WriteLog('has_child','notice');

            $ret =$zip->get_packages($data);

            $packages=$this->task->get_packages_info(WPVIVID_BACKUP_TYPE_MERGE);
            if($packages===false)
            {
                $packages=$this->task->set_packages_info(WPVIVID_BACKUP_TYPE_MERGE,$ret['packages']);
            }

            define(PCLZIP_TEMPORARY_DIR,$ret['temp_dir']);

            $result['result']=WPVIVID_SUCCESS;
            $result['files']=array();

            foreach ($packages as $package)
            {
                if(!empty($package['files'])&&$package['backup']==false)
                {
                    $zip_ret=$zip->_zip($package['path'],$package['files'], $data,$package['json']);
                    if($zip_ret['result']==WPVIVID_SUCCESS)
                    {
                        $result['files'][] = $zip_ret['file_data'];
                        $package['backup']=true;
                        $this->task->update_packages_info(WPVIVID_BACKUP_TYPE_MERGE,$package);
                    }
                    else
                    {
                        $result=$zip_ret;
                        break;
                    }
                }else {
                    continue;
                }
            }
            $wpvivid_pulgin->wpvivid_log->WriteLog('Compressing '.$data['key'].' completed','notice');
            return $result;
        }
        else if(isset($data['wp_plugin']))
        {
            global $wpvivid_pulgin;
            $wpvivid_pulgin->wpvivid_log->WriteLog('Start compressing '.$data['key'],'notice');

            $ret =$zip->get_plugin_packages($data);

            $packages=$this->task->get_packages_info(WPVIVID_BACKUP_TYPE_PLUGIN);
            if($packages===false)
            {
                $packages=$this->task->set_packages_info(WPVIVID_BACKUP_TYPE_PLUGIN,$ret['packages']);
            }

            define(PCLZIP_TEMPORARY_DIR,$ret['temp_dir']);

            $result['result']=WPVIVID_SUCCESS;
            $result['files']=array();

            foreach ($packages as $package)
            {
                if(!empty($package['files'])&&$package['backup']==false)
                {
                    $zip_ret=$zip->_zip($package['path'],$package['files'], $data,$package['json']);
                    if($zip_ret['result']==WPVIVID_SUCCESS)
                    {
                        $result['files'][] = $zip_ret['file_data'];
                        $package['backup']=true;
                        $this->task->update_packages_info(WPVIVID_BACKUP_TYPE_PLUGIN,$package,$zip_ret['file_data']);
                    }
                    else
                    {
                        $result=$zip_ret;
                        break;
                    }
                }else {
                    continue;
                }
            }
            $wpvivid_pulgin->wpvivid_log->WriteLog('Compressing '.$data['key'].' completed','notice');
            return $result;
        }
        else
        {
            $result =$zip->compress($data);

            if(isset($data['dump_db']))
            {
                foreach ($data['files'] as $sql_file)
                {
                    @unlink($sql_file);
                }
            }
        }

        return $result;
        */
    }

    public function cleanup()
    {
        $files=$this->task->get_need_cleanup_files();

        foreach ($files as $file)
        {
            if(file_exists($file)) {
                global $wpvivid_pulgin;
                $wpvivid_pulgin->wpvivid_log->WriteLog('Cleaned up file, filename: '.$file,'notice');
                @unlink($file);
            }
        }
    }

    public function clean_backup()
    {
        $path = WP_CONTENT_DIR.DIRECTORY_SEPARATOR.WPvivid_Setting::get_backupdir();
        $handler=opendir($path);
        while(($filename=readdir($handler))!==false)
        {
            if(preg_match('#'.$this->task->get_id().'#',$filename))
            {
                @unlink($path.DIRECTORY_SEPARATOR.$filename);
            }
        }
        @closedir($handler);
        WPvivid_tools::clearcache($this->task->get_prefix());
    }

    public function clean_remote_backup($remotes,$files)
    {
        $remote_option=array_shift($remotes);

        if(!is_null($remote_option))
        {
            global $wpvivid_pulgin;

            $remote=$wpvivid_pulgin->remote_collection->get_remote($remote_option);
            $remote ->cleanup($files);
        }
    }
}