<?php

/* Add metaboxes to the theme */

/**
 * Register meta box(es).
 */
function npchild_register_meta_boxes() {
    /* This metabox will be available only for the designated home page */
    if (get_page_template_slug() == 'templates/template-home.php'):
        add_meta_box( 'show_video_section', __( 'Page Options', 'npchild' ), 'show_video_section_callback', 'page', 'side', 'high' );
    endif;
}
add_action( 'add_meta_boxes', 'npchild_register_meta_boxes' );
 
/**
 * Meta box display callback.
 *
 * @param WP_Post $post Current post object.
 */

function show_video_section_callback( $post ) {
    //Adding Nonce
    wp_nonce_field( basename( __FILE__ ), 'show_video_section_nonce' );
    
    $show_video_optin = get_post_meta($post->ID,'show_video_optin',true );
?>
	<input type="checkbox" id="show_video_optin" name="show_video_optin" value=<?php echo $show_video_optin;?> <?php checked("on", $show_video_optin, "checked");?> />Show Video / Optin Section <br />

<?php
}
 
/**
 * Save meta box content.
 *
 * @param int $post_id Post ID
 */
function show_video_section_save( $post_id ) {
    
    if ( !isset( $_POST['show_video_section_nonce'] ) || !wp_verify_nonce( $_POST['show_video_section_nonce'], basename( __FILE__ ) ) ){
		return;
	}
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return $post_id;
    }
    if ( ! current_user_can( 'edit_post', $post_id ) ){
	    return;
    }
    if (get_page_template_slug() !== 'templates/template-home.php'){
        return;
    }
    if(isset($_POST["show_video_optin"])){
        update_post_meta($post_id, "show_video_optin", "on");
    }else{
        update_post_meta($post_id, "show_video_optin", "off");
    }
}
add_action( 'save_post', 'show_video_section_save' );

?>